package com.company.demo.component;

import com.vaadin.flow.component.*;
import com.vaadin.flow.component.AbstractField.ComponentValueChangeEvent;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.shared.Registration;
import io.jmix.core.MetadataTools;
import io.jmix.flowui.UiComponents;
import io.jmix.flowui.component.HasRequired;
import io.jmix.flowui.component.checkbox.JmixCheckbox;
import io.jmix.flowui.component.textfield.TypedTextField;
import io.jmix.flowui.data.SupportsValueSource;
import io.jmix.flowui.data.ValueSource;
import io.jmix.flowui.kit.component.HasTitle;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;


public class MasterDataField<V> extends Composite<VerticalLayout>
        implements HasValue<ComponentValueChangeEvent<MasterDataField<V>, V>, V>,
        SupportsValueSource<V>, HasRequired, HasTitle, HasLabel, HasSize,
        ApplicationContextAware,
        InitializingBean {

    protected ApplicationContext applicationContext;
    protected UiComponents uiComponents;
    protected MetadataTools metadataTools;

    protected TypedTextField<V> textField;
    protected JmixCheckbox checkbox;

    protected MasterDataFieldDelegate<MasterDataField<V>, V, V> fieldDelegate;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) {
        this.applicationContext = applicationContext;
    }

    @Override
    public void afterPropertiesSet() {
        uiComponents = applicationContext.getBean(UiComponents.class);
        metadataTools = applicationContext.getBean(MetadataTools.class);
        fieldDelegate = applicationContext.getBean(MasterDataFieldDelegate.class, this);
    }

    @Override
    public void setValue(V value) {
        if (checkbox != null) {
            checkbox.setValue((Boolean) value);
        } else if (textField != null) {
            textField.setTypedValue(value);
        }
    }

    @Override
    public V getValue() {
        if (checkbox != null) {
            return (V) checkbox.getValue();
        } else if (textField != null) {
            return textField.getTypedValue();
        } else {
            return null;
        }
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    @Override
    public Registration addValueChangeListener(ValueChangeListener<? super ComponentValueChangeEvent<MasterDataField<V>, V>> listener) {
        ComponentEventListener componentListener = event -> {
            ComponentValueChangeEvent<MasterDataField<V>, V> valueChangeEvent = (ComponentValueChangeEvent<MasterDataField<V>, V>) event;
            listener.valueChanged(valueChangeEvent);
        };
        return ComponentUtil.addListener(this,
                ComponentValueChangeEvent.class, componentListener);
    }

    @Override
    public void setReadOnly(boolean readOnly) {
        // implement
    }

    @Override
    public boolean isReadOnly() {
        // implement
        return false;
    }

    @Override
    public void setRequiredIndicatorVisible(boolean requiredIndicatorVisible) {
        // implement
    }

    @Override
    public boolean isRequiredIndicatorVisible() {
        // implement
        return false;
    }

    @Override
    public String getRequiredMessage() {
        return "";
    }

    @Override
    public void setRequiredMessage(String requiredMessage) {
        // implement
    }

    @Override
    public ValueSource<V> getValueSource() {
        return fieldDelegate.getValueSource();
    }

    @Override
    public void setValueSource(ValueSource<V> valueSource) {
        fieldDelegate.setValueSource(valueSource);

        getContent().removeAll();
        if (valueSource == null) {
            return;
        }
        if (Boolean.class.isAssignableFrom(valueSource.getType())) {
            initCheckbox();
        } else {
            initTypedTextField();
        }
    }

    @SuppressWarnings("unchecked")
    private void initCheckbox() {
        checkbox = uiComponents.create(JmixCheckbox.class);
        checkbox.addValueChangeListener(event -> {
            ComponentValueChangeEvent<MasterDataField<V>, V> changeEvent =
                    new ComponentValueChangeEvent<>(this, this, (V) event.getOldValue(), true);

            ComponentUtil.fireEvent(this, changeEvent);
        });
        getContent().add(checkbox);
    }

    @SuppressWarnings("unchecked")
    private void initTypedTextField() {
        textField = uiComponents.create(TypedTextField.class);
        textField.addTypedValueChangeListener(event -> {
            ComponentValueChangeEvent<MasterDataField<V>, V> changeEvent =
                    new ComponentValueChangeEvent<>(this, this, event.getOldValue(), true);

            ComponentUtil.fireEvent(this, changeEvent);
        });
        getContent().add(textField);
    }
}
